/*==================================================================
  ==================================================================
  ==	XMusic -- An Interactive Interface to Csound		  ==
  ==       Copyright (C) 1988 by George D. Drapeau                ==
  ==                  All Rights Reserved                         ==
  ==================================================================
  ==================================================================
*/
#include <X/Xlib.h>
#include <X/Xkeyboard.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "defs.h"
#include "externs.h"



/*############################################################################
  FUNCTION	DoXEventLoop
  DESCRIPTION:
  	This is the main function for the X part of Music.  It is responsible
	for catching X events sent to this client (i.e., program) by the
	X server.  The function calls other functions to open the display,
	open any fonts needed by Music, define windows to the X server, and
	so on.  After doing that, the function goes into an infinite while
	loop typical of X clients, waiting until an X event is received.
	When an event is found, it is taken from the event queue and the
	appropriate function is called, depending on the type of event
	received.
	This function will never exit normally; the ExitPgm function is
	responsible for exiting Music.
*/


DoXEventLoop()
{
  
  if (XOpenDisplay("")==NULL)					    /* This X call must be done before any other X calls!! */
    {
      fprintf(stderr,"%s: Could not open Display \n", WhoAmI);	    /* Couldn't open the display for some reason, so... */
      exit(1);							    /* ...report the error and exit with an error code */
    }
  GetXDefaults();						    /* Get user preferences for Music */
  CreateWindows();						    /* Create all windows used by Music */
  GetXResources();						    /* Set up cursors, icons, etc needed by this program */
  GetToolPixmaps();						    /* Fill in the tools' Pixmap id's now that they're ready */
  PrintToolList();						    /* Print list of tools read in from file */
  changesMade = 0;						    /* Set flag saying nothing has been done yet */
  
  XSelectInput(musicWin,					    /* Specify which windows listen for which events */
	       ExposeWindow | ButtonPressed | KeyPressed | ExposeRegion);
  XSelectInput(helpWin,
	       ExposeWindow | ButtonPressed | KeyPressed);
  XSelectInput(exitWin,
	       ExposeWindow | ButtonPressed);
  XSelectInput(toolWin,
	       ExposeWindow | ButtonPressed | KeyPressed);
  XSelectInput(iconWin,
	       ExposeWindow);
  XSelectInput(inputWin,
	       ExposeWindow | ButtonPressed | KeyPressed);
  XSelectInput(infoWin,
	       ExposeWindow | ButtonPressed | KeyPressed);
  
  XMapWindow(musicWin);						    /* Get ready to draw the Music window  */
  XMapWindow(toolWin);						    /* Get ready to draw the tool window */
  while (1)							    /* Process events forever, like all X clients */
    {
      XEvent	theEvent;					    /* Space to store info about events received */
      XSync(0);							    /* Catch up with the X server */
      XMaskEvent(ExposeWindow | ExposeRegion | KeyPressed	    /* Looking for these events in any window that... */
		 |ButtonPressed, &theEvent);			    /* ... an 'XSelectInput' was called for */
      switch (theEvent.type)					    /* What type of an event was found? */
	{
	case ButtonPressed:
	  DoButtonEvent((XKeyOrButtonEvent *)&theEvent);
	  break;
	case KeyPressed:
	  DoKeyEvent((XKeyOrButtonEvent *)&theEvent);
	  break;
	case ExposeWindow:
	  DoExposeWinEvent((XExposeWindowEvent *)&theEvent);
	  break;
	case ExposeRegion:
	  DoExposeRegEvent((XExposeRegionEvent *)&theEvent);
	  break;
	}							    /* end switch */
    }								    /* end while... */
}								    /* end function DoXEventLoop */




DoButtonEvent(event)
     XKeyOrButtonEvent	*event;
{
  switch (event->detail)
    {
    case LeftButton:
      if (event->window == toolWin)
	DoSelectEvent(event);
      if (event->window == musicWin)
	ConnectTools();
      if (event->window == infoWin)
	ChangeArg(event);
      break;
    case MiddleButton:
      MoveTool();
      break;
    case RightButton:
      DisconnectTools();
      break;
    }								    /* end switch */
}
								    /* end function DoButtonEvent */



DoKeyEvent(event)
     XKeyOrButtonEvent	*event;
{
  short	keyCode;
  char	*key, c;
  int	nBytes;
  
  keyCode = event->detail & 0x00FF;				    /* Key press info is in lower byte only */
  key=XLookupMapping((XKeyPressedEvent *)event, &nBytes);	    /* Yes, what key did s/he type? */
  if (nBytes == 0)						    /* User typed a key not in the keyboard mapping... */
      return;							    /* ...go to next iteration of the while loop */
  c = *key;							    /* Store character that X function returned */
  if (isalnum(c))						    /* Did the user type an alphanumeric? */
    switch (c)
      {
      case 'q':							    /* Did the user type q to quit? */
      case 'Q':
	ExitPgm();						    /* Yes, ask the user to confirm this choice */
	break;
      case 'h':							    /* Did the user type h to get help? */
      case 'H':
	DoHelp();						    /* Yes, show or hide the Help window */
	break;
      case 'i':							    /* Does the user want info on a particular tool? */
      case 'I':
	DoToolInfo();
	break;
      case 'r':
      case 'R':
	RemoveTool();
	break;
      case 's':
      case 'S':
	WriteDiagram();
	break;
      case 'w':
      case 'W':
	WriteOrchFile();
	break;
      default:
	break;
      }								    /* end switch (c)... */
}								    /* end function DoKeyEvent */





DoExposeWinEvent(event)
     XExposeWindowEvent	*event;
{
  if (event->window == musicWin)
    {
      DrawMusicWin();
      return;
    }
  if (event->window == helpWin)
    {
      DrawHelpWin();
      return;
    }
  if (event->window == toolWin)
    {
      DrawToolWin();
      return;
    }
  if (event->window == iconWin)
    {
      DrawIconWin();
      return;
    }
  if (event->window == infoWin)
    {
      GetToolInfo(lastTool);
      DrawInfoWin(lastTool);
      return;
    }
}
								    /* end function DoExposeWinEvent */


DoExposeRegEvent(event)
     XExposeRegionEvent	*event;
{
  if (event->window == musicWin)				    /* Was the exposure event caught in the Music window? */
    {
      DrawPixmap(musicPixmap,event);				    /* Yes, redraw the exposed region */
    }
}								    /* end function DoExposeRegEvent */



DoSelectEvent(event)
     XKeyOrButtonEvent	*event;
{
  int	whichTool;
  
  if (event->window == toolWin)
    {
      whichTool = SelectTool(event);
      PlaceTool(whichTool);
      PrintToolList();
      return;
    }
}								    /* end function DoSelectEvent */
